from datetime import datetime
from pyspark.sql import SparkSession
from sigma_dq.helper.sigma_dq_generate_dq_report import sigma_dq_generate_dq_report


spark = SparkSession.builder.master("local[*]") \
    .appName('sigma_dq') \
    .getOrCreate()

# sigma_dq_core_endsWith("MATERIAL_NO","75",target_table)
# sigma_dq_check_endsWith('JDE_UDC_PURCHASE_DOCUMENT_TYPE_DESC','DT',target_table,'Incremental',meta={'dq_master_id' : 1})
def sigma_dq_check_ends_with(target_column,string_check,target_table,execution_type = '',
from_date=0, to_date=0,meta={}):
    column = target_column
    dq_rule = 'endsWith'

    if execution_type == 'Incremental':
        str_sql = f"select {target_column}, case when {target_column} like '%{string_check}' then \
            'PASS' else 'FAIL' end as DQ_Status from {target_table} WHERE UPDATE_RUN_TS = \
                (select MAX(UPDATE_RUN_TS) from {target_table}) "
    elif execution_type == "date_range":
        from_timestamp = datetime.fromtimestamp(from_date)
        from_date_str = from_timestamp.strftime( "%Y-%m-%d")
        to_timestamp = datetime.fromtimestamp(to_date)
        to_date_str = to_timestamp.strftime( "%Y-%m-%d")
        between_condition = \
            f' WHERE cast(update_run_ts as string) between "{from_date_str}" AND "{to_date_str}"'
        str_sql = "select "+target_column+",case when "+target_column+" like '%"+string_check+ \
            "' then 'PASS' else 'FAIL' end as DQ_Status from "+target_table + between_condition
    else:
        str_sql = f"select {target_column}, case when {target_column} like '%{string_check}' then \
            'PASS' else 'FAIL' end as DQ_Status from {target_table} "

    dq_apply_column_data = spark.sql(str_sql)
    dq_report = sigma_dq_generate_dq_report(dq_apply_column_data,column,dq_rule)

    return dq_report



from datetime import datetime
from pyspark.sql import SparkSession
from sigma_dq.helper.sigma_dq_generate_dq_report import sigma_dq_generate_dq_report


spark = SparkSession.builder.master("local[*]") \
    .appName('sigma_dq') \
    .getOrCreate()

# sigma_dq_check_fieldCompare(target_column,api_response,target_table)
# '%<>%80'
def sigma_dq_check_field_compare_with_value(target_column,api_response,target_table,execution_type='',from_date=0,to_date=0,meta={}):
    column = target_column
    dq_rule = 'fieldCompare_zero'
    source_table_with_column = target_table+'.'+target_column
    source_table = target_table
    operator = ''.join(api_response.split('%')[1:-1])
    value = ''.join(api_response.split('%')[::-3])

    try:
        value = float(value)
    except:
        return("Insert valid number")
    else:
        value = float(value)

    if operator == '>':  # Greater than 0
        operator = '> ' + str(value)
        str_sql = "select "+source_table_with_column+", CASE WHEN " +source_table_with_column+" "+\
            operator+" THEN 'PASS' ELSE 'FAIL' END as DQ_Status from " + source_table +"  "

    elif operator == '>':  # Greater than equal to 0
        operator = '> '+ str(value)
        str_sql = "select "+source_table_with_column+", CASE WHEN " +source_table_with_column+" "+\
            operator+" THEN 'PASS' ELSE 'FAIL' END as DQ_Status from " + source_table +"  "

    elif operator == '<':  # Less than 0
        operator = '< '+ str(value)
        str_sql = "select "+source_table_with_column+", CASE WHEN " +source_table_with_column+" "+\
            operator+" THEN 'PASS' ELSE 'FAIL' END as DQ_Status from " + source_table+"  "

    elif operator == '<=':  # Less than equal to 0
        operator = '<= '+ str(value)
        str_sql = "select "+source_table_with_column+", CASE WHEN " +source_table_with_column+" "+\
            operator+" THEN 'PASS' ELSE 'FAIL' END as DQ_Status from " + source_table+"  "

    elif operator == '<>':  # Not equal to 0
        operator = '<> '+ str(value)
        str_sql = "select "+source_table_with_column+", CASE WHEN " +source_table_with_column+" "+\
            operator+" THEN 'PASS' ELSE 'FAIL' END as DQ_Status from " + source_table+"  "

    elif operator == '=':  # Not equal to 0
        operator = '= '+ str(value)
        str_sql = "select "+source_table_with_column+", CASE WHEN " +source_table_with_column+" "+\
            operator+" THEN 'PASS' ELSE 'FAIL' END as DQ_Status from " + source_table+"  "

    else:
        return "Error please check the input dqRule:sigma_dq_check_fieldCompare_with_value"

    if execution_type == 'Incremental': 
        str_sql += " where UPDATE_RUN_TS = (select MAX(UPDATE_RUN_TS) from "+target_table+ \
            ") OR dqAction = 'NA'"
    elif execution_type == "date_range":
        from_timestamp = datetime.fromtimestamp(from_date)
        from_date_str = from_timestamp.strftime( "%Y-%m-%d")
        to_timestamp = datetime.fromtimestamp(to_date)
        to_date_str = to_timestamp.strftime( "%Y-%m-%d")
        between_condition = \
            f' WHERE cast(update_run_ts as string) between "{from_date_str}" AND "{to_date_str}"'
        str_sql += between_condition

    dq_apply_column_data = spark.sql(str_sql)
    dq_report = sigma_dq_generate_dq_report(dq_apply_column_data,column,dq_rule)

    return dq_report



def sigma_dq_generate_dq_report(dq_apply_column_data,column,dq_rule,meta={}):
    dq_report = {}
    dq_report['column'] = column
    dq_report['rule'] = dq_rule
    print(f"column: {column}, RuleName: {dq_rule}")
    dq_report['total_rows_checked'] = dq_apply_column_data.count()
    total_dq_pass_count = \
        dq_apply_column_data.filter(dq_apply_column_data.DQ_Status == 'PASS').count()
    dq_report['total_rows_failed'] = dq_apply_column_data.filter(
        dq_apply_column_data.DQ_Status == 'FAIL').count()
    if dq_report['total_rows_failed'] > 0:
        print("inside failed")
        dq_report['success'] = False
        dq_report['failed_percent'] = \
            (dq_report['total_rows_failed']/dq_report['total_rows_checked'])*100
        raw_fail_list = dq_apply_column_data.select(dq_apply_column_data[column]). \
            filter(dq_apply_column_data.DQ_Status == 'FAIL'). \
            rdd.flatMap(lambda x: x). \
            collect()
        dq_report['failed_values'] = raw_fail_list
    elif dq_report['total_rows_checked'] < 1:
        dq_report['success'] = False
        dq_report['successResult'] = 'Table is empty'
        dq_report['passed_percent'] = []
        dq_report['failed_values'] = []
        dq_report['failed_percent'] = []

    elif dq_report['total_rows_failed'] == dq_report['total_rows_checked']:
        dq_report['success'] = False
        dq_report['failed_values'] = []
        dq_report['failed_percent'] = []

    elif dq_report['total_rows_failed'] == dq_report['total_rows_checked']:
        dq_report['success'] = False
        dq_report['failed_percent'] = \
            (dq_report['total_rows_failed']/dq_report['total_rows_checked'])*100
        dq_report['failed_values'] = []
    else:
        dq_report['success'] = True
        dq_report['passed_percent'] = (total_dq_pass_count/dq_report['total_rows_checked'])*100

    dq_report['meta'] = meta
    return dq_report



from datetime import datetime
from pyspark.sql import SparkSession
from sigma_dq.helper.sigma_dq_generate_dq_report import sigma_dq_generate_dq_report


spark = SparkSession.builder.master("local[*]") \
    .appName('sigma_dq') \
    .getOrCreate()

#sigma_dq_check_isInList('PRMSDDLVDT',target_table,'')
def sigma_dq_check_is_in_list(target_column,list_of_values,target_table,execution_type='',
from_date=0,to_date=0,meta={}):
    column = target_column
    dq_rule = 'isInList'

    if execution_type == 'Incremental':
        str_sql = "select "+target_column+",case when "+target_column+"  in \
            ('"+"','".join(list_of_values)+"') then 'PASS' else 'FAIL' end as DQ_Status from  "+\
                target_table + ' WHERE UPDATE_RUN_TS = (select MAX(UPDATE_RUN_TS) from '+\
                    target_table+') '
    elif execution_type == "date_range":
        from_timestamp = datetime.fromtimestamp(from_date)
        from_date_str = from_timestamp.strftime( "%Y-%m-%d")
        to_timestamp = datetime.fromtimestamp(to_date)
        to_date_str = to_timestamp.strftime( "%Y-%m-%d")
        between_condition = \
            f' WHERE cast(update_run_ts as string) between "{from_date_str}" AND "{to_date_str}"'
        str_sql = "select "+target_column+",case when "+target_column+" in \
            ('"+"','".join(list_of_values)+"') then 'PASS' else 'FAIL' end as DQ_Status from "+\
                target_table + between_condition
    else:
        str_sql = "select "+target_column+",case when "+target_column+" in \
            ('"+"','".join(list_of_values)+"') then 'PASS' else 'FAIL' end as DQ_Status from  "+\
                target_table + " "

    dq_apply_column_data = spark.sql(str_sql)
    dq_report = sigma_dq_generate_dq_report(dq_apply_column_data,column,dq_rule)

    return dq_report



from datetime import datetime
from pyspark.sql import SparkSession
from sigma_dq.helper.sigma_dq_generate_dq_report import sigma_dq_generate_dq_report


spark = SparkSession.builder.master("local[*]") \
    .appName('sigma_dq') \
    .getOrCreate()

"""sigma_dq_check_isNull('PRMSDDLVDT',target_table,'',meta = {})"""
def sigma_dq_check_is_null(target_column,target_table,execution_type='',from_date=0,to_date=0):
    column = target_column
    dq_rule = 'isNull'

    if execution_type == 'Incremental':
        str_sql = "select "+target_column+",case when "+target_column+" is not null AND TRIM("+\
            target_column+") <> ''  then 'PASS' else 'FAIL' end as DQ_Status from  "+target_table \
                + ' WHERE UPDATE_RUN_TS = (select MAX(UPDATE_RUN_TS) from '+target_table+') '

    elif execution_type == "date_range":
        from_timestamp = datetime.fromtimestamp(from_date)
        from_date_str = from_timestamp.strftime( "%Y-%m-%d")
        to_timestamp = datetime.fromtimestamp(to_date)
        to_date_str = to_timestamp.strftime( "%Y-%m-%d")
        between_condition = \
            f' WHERE cast(update_run_ts as string) between "{from_date_str}" AND "{to_date_str}"'
        str_sql = "select "+target_column+",case when "+target_column+" is not null AND TRIM("+\
            target_column+") <> ''  then 'PASS' else 'FAIL' end as DQ_Status from  "+target_table \
                + between_condition

    else:
        str_sql = "select "+target_column+",case when "+target_column+" is not null AND TRIM("+\
            target_column+") <> ''  then 'PASS' else 'FAIL' end as DQ_Status from  "+ \
                target_table + ' '

    dq_apply_column_data = spark.sql(str_sql)
    dq_report = sigma_dq_generate_dq_report(dq_apply_column_data,column,dq_rule)

    return dq_report
